@@ -15,51 +15,119 @@
 
 from __future__ import annotations
 
-import re
+from re import Match, search, compile as _compile, match
 from abc import ABC
+from copy import deepcopy
+
+from typing import TYPE_CHECKING
+
+from typing import Generic, TypeVar, cast
+
+if TYPE_CHECKING:
+    from typing import Optional, ClassVar, Type, Dict, List
+
+T_ChangelogFormater = TypeVar("T_ChangelogFormater", bound="ChangelogFormater")
 
 
-def ChangeLogFormaterRecordType(Klass: type) -> type:
-    """Decorator helper function to register interface implementation in factory
+class _ChangelogFormatersCtx(Generic[T_ChangelogFormater]):
+    """Storage class that manage Formaters"""
+
+    def __init__(self) -> None:
+        """Storage class init method"""
+        self._savedFormaterList: set[Type[T_ChangelogFormater]] = set()
+
+    def add(self, record: Type[T_ChangelogFormater]) -> None:
+        """Add a Formater to the storage class
+        Args:
+            record: the Formater class to be added
+        """
+        self._savedFormaterList.add(record)
+
+    def remove(self, record: Type[T_ChangelogFormater]) -> None:
+        """Remove a Formater from the storage class
+        Args:
+            record: the Formater class to be removed
+        """
+        self._savedFormaterList.remove(record)
+
+    def reset(self) -> None:
+        """Reset the storage class"""
+        self._savedFormaterList = set()
+
+    def get(self) -> set[Type[T_ChangelogFormater]]:
+        """Get the storage data set
+        Returns:
+            The internal storage class (set)
+        """
+        return self._savedFormaterList
+
+    def __copy__(self) -> _ChangelogFormatersCtx[T_ChangelogFormater]:
+        """Copy the class"""
+        cls = self.__class__
+        result = cls.__new__(cls)
+        result._savedFormaterList = self._savedFormaterList
+        return result
+
+    def __deepcopy__(self, memo: Dict[int, object]) -> _ChangelogFormatersCtx[T_ChangelogFormater]:
+        """Deep-Copy the class
+        Args:
+            memo: __deepcopy__ interface impl"""
+        result = self.__copy__()
+        memo[id(self)] = result
+        result._savedFormaterList = self._savedFormaterList.copy()
+        return result
+
+
+def ChangelogFormaterRecordType(Klass: Type[T_ChangelogFormater]) -> Type[T_ChangelogFormater]:
+    """Decorator function that registers formater implementation in factory
     Args:
         Klass: class to register in the factory
     Returns:
         untouched class"""
-    ChangeLogFactory.ar_FormaterKlass.add(Klass)
+    ChangelogFactory.RegisterBaseFormater(Klass)
     return Klass
 
 
-class ChangeLogFormater(ABC):
-
-    prefix: str = "^\s+"
-    title: str = "Others :"
-    keywords: list[str] = []
-    priority: int = 0
-
-    _lines: list[str] = []
-
-    def __init__(self):
-        """ChangeLogFormater class constructor
-
-        This class is the base formater class.
-
-        This class is for:
-
-        - classifying message: CheckLine() and CheckLine_keywords()
-        - storing lines: Clear() and PushLine()
-        - returning the formated output: Render() and RenderLines()
+def _ChangelogFormaterRecordType(Klass: Type[T_ChangelogFormater]) -> Type[T_ChangelogFormater]:
+    """Internal decorator function that registers formater implementation in factory
+    Args:
+        Klass: class to register in the factory
+    Returns:
+        untouched class"""
+    cast(ChangelogFactory[ChangelogFormater], ChangelogFactory).ar_SavedFormaterKlass.add(Klass)
+    return ChangelogFormaterRecordType(Klass)
 
 
-        /// warning
-        this class does not aim to be instantiated by user.
-        ///
+class ChangelogFormater(ABC):
+    """ChangelogFormater class
 
-        """
-        self._lines: list[str] = []
+    This class is the formater base class.
+
+    This class is for:
+
+    - classifying message: CheckLine() and CheckLine_keywords()
+    - storing lines: Clear() and PushLine()
+    - returning the formated output: Render() and RenderLines()
+
+
+    /// warning
+    this class does not aim to be instantiated by user.
+    ///
+    """
+
+    prefix: ClassVar[Optional[str]] = None
+    title: ClassVar[Optional[str]] = None
+    keywords: ClassVar[Optional[list[str]]] = None
+    priority: ClassVar[int] = 0
+    _lines: List[None | str] = []
+
+    def __init__(self) -> None:
+        """ChangelogFormater class constructor"""
+        self._lines: List[None | str] = []
 
     def Clear(self) -> None:
         """Clear the formater content"""
-        self._lines: list[str] = []
+        self._lines = []
 
     def PushLine(self, ChangelogString: str) -> None:
         """Push a new line in the formater
@@ -91,7 +159,7 @@ class ChangeLogFormater(ABC):
         return full_lines
 
     @classmethod
-    def CheckLine(cls, content: str) -> re.Match:
+    def CheckLine(cls, content: str) -> None | Match[str]:
         """Check if a line match the current formater (lazy identification)
 
         /// warning
@@ -104,7 +172,7 @@ class ChangeLogFormater(ABC):
         Returns:
             match object
         """
-        regex = re.compile(r"^(?:-\s+)?(?:{0})(?:\((.*)\))?(?::)(?:\s*)([^\s].+)".format(cls.prefix))
+        regex = _compile(rf"^(?:-\s+)?(?:{cls.prefix})(?:\((.*)\))?(?::)(?:\s*)([^\s].+)")
         _match = regex.match(content)
         return _match
 
@@ -120,59 +188,95 @@ class ChangeLogFormater(ABC):
             True if a keyword has matched, False otherwise
         """
         keyword_list = cls.keywords
-        for _keyword in keyword_list:
-            if (_keyword != "") and re.search(_keyword, content):
-                return True
+        if keyword_list:
+            for _keyword in keyword_list:
+                if _keyword and search(_keyword, content):
+                    return True
         return False
 
 
-class ChangeLogFactory:
+class ChangelogFactory(Generic[T_ChangelogFormater]):
     """The main changelog class"""
 
-    ar_FormaterKlass: set[type[ChangeLogFormater]] = set()
-    ar_Formater: dict[ChangeLogFormater] = dict()
+    ar_SavedFormaterKlass: ClassVar[_ChangelogFormatersCtx[ChangelogFormater]] = _ChangelogFormatersCtx[ChangelogFormater]()
+    ar_FormaterKlass: _ChangelogFormatersCtx[T_ChangelogFormater] = _ChangelogFormatersCtx[T_ChangelogFormater]()
+
+    ar_Formater: Dict[str, T_ChangelogFormater] = {}
     checkCommentPattern: str = r"^[ \t]*(?:\/\/|#)"
 
-    def __init__(self, ChangelogString: None | str = None):
-        """Main ChangeLogFormater class constructor
+    def __init__(self, ChangelogString: Optional[str] = None) -> None:
+        """Main ChangelogFormater class constructor
 
         Args:
-            ChangelogString: optionnal input string to start with
+            ChangelogString: optional input string to be processed
         """
-        for FormaterKlass in ChangeLogFactory.ar_FormaterKlass:
+        self.ar_Formater: Dict[str, T_ChangelogFormater] = {}
+        self.ar_FormaterKlass = deepcopy(type(self).ar_FormaterKlass)
+
+        for FormaterKlass in self.ar_FormaterKlass.get():
             self.ar_Formater[FormaterKlass.__name__] = FormaterKlass()
 
+        # missing mypy coverage here because of internal bad isinstance() handling
+        # could be fixed using 'type(ChangelogString) is str' but then quality check will bad
+        # so let quality advise
         if isinstance(ChangelogString, str):
             self.ProcessFullChangelog(ChangelogString)
 
-    def RegisterFormater(self, FormaterKlass: ChangeLogFormater) -> None:
-        """Register a new formater
+    def ResetFormaterList(self) -> ChangelogFactory[T_ChangelogFormater]:
+        """Reset the formater class list to original (Instance wise)
+        Returns:
+            self for convenience
+        """
+        self.ar_FormaterKlass: T_ChangelogFormater = deepcopy(
+            cast(_ChangelogFormatersCtx[T_ChangelogFormater], ChangelogFactory.ar_SavedFormaterKlass)
+        )
+        self.ar_Formater = {}
+        for FormaterKlass in self.ar_FormaterKlass.get():
+            self.ar_Formater[FormaterKlass.__name__] = FormaterKlass()
+        return self
+
+    @classmethod
+    def RegisterBaseFormater(cls, FormaterKlass: Type[T_ChangelogFormater]) -> None:
+        """Register a new formater in the current instance
+
+        Args:
+            FormaterKlass: class of the formater to be added
+        """
+        cls.ar_FormaterKlass.add(FormaterKlass)
+
+    @classmethod
+    def ResetBaseFormaterList(cls) -> None:
+        """Reset the formater class list to original (BaseClass wise)"""
+        cls.ar_FormaterKlass = deepcopy(cls.ar_SavedFormaterKlass)
+
+    def RegisterFormater(self, FormaterKlass: Type[T_ChangelogFormater]) -> ChangelogFactory[T_ChangelogFormater]:
+        """Register a new formater in the current instance
 
         Args:
             FormaterKlass: class of the formater to be added
         Returns:
-            self class for convenience
+            self for convenience
         """
         self.ar_FormaterKlass.add(FormaterKlass)
         self.ar_Formater[FormaterKlass.__name__] = FormaterKlass()
         return self
 
-    def unRegisterFormater(self, FormaterKlass: ChangeLogFormater) -> None:
-        """Register a new formater
+    def unRegisterFormater(self, FormaterKlass: Type[T_ChangelogFormater]) -> ChangelogFactory[T_ChangelogFormater]:
+        """unRegister a new formater in the current instance
 
         Args:
-            FormaterKlass: class of the formater to be removed
+            FormaterKlass: class of the formater to be dropped
         Returns:
-            self class for convenience
+            self for convenience
         """
         self.ar_FormaterKlass.remove(FormaterKlass)
         del self.ar_Formater[FormaterKlass.__name__]
         return self
 
-    def Clear(self) -> ChangeLogFactory:
+    def Clear(self) -> ChangelogFactory[T_ChangelogFormater]:
         """Clear internal memory
         Returns:
-            self class for convenience
+            self for convenience
         """
         for formater in self.ar_Formater.values():
             formater.Clear()
@@ -185,14 +289,16 @@ class ChangeLogFactory:
         If a matching formater is found, line is inserted.
 
         Args:
-            RawChangelogLine: line to parse
+            RawChangelogLine: line to process
         Returns:
             True if successfully matched, False otherwise
         """
         for formater in sorted(self.ar_Formater.values(), key=lambda x: x.priority):
-            content = formater.CheckLine(RawChangelogLine)
-            if content is not None:
-                formater.PushLine(content.group(2))
+            content: Optional[Match[str]] = formater.CheckLine(RawChangelogLine)
+            # missing mypy coverage here because of internal bad isinstance() handling AND Match type
+            if isinstance(content, Match) and (len(content.groups()) == 2):
+                res: str = content.group(2)
+                formater.PushLine(res)
                 return True
 
         return False
@@ -204,7 +310,7 @@ class ChangeLogFactory:
         If a matching formater is found, line is inserted.
 
         Args:
-            RawChangelogLine: line to parse
+            RawChangelogLine: line to process
         Returns:
             True if successfully matched, False otherwise
         """
@@ -213,34 +319,37 @@ class ChangeLogFactory:
                 formater.PushLine(RawChangelogLine)
                 return True
 
-        self.ar_Formater[ChangeLogFormater_others.__name__].PushLine(RawChangelogLine)
+        self.ar_Formater[ChangelogFormater_others.__name__].PushLine(RawChangelogLine)
         return False
 
-    def ProcessFullChangelog(self, RawChangelogMessage: str) -> ChangeLogFactory:
+    def ProcessFullChangelog(self, RawChangelogMessage: str) -> ChangelogFactory[T_ChangelogFormater]:
         """Process all input lines
 
         This function handles the main 2-round changes search algo.
         It takes care of search-order and automatically skip any non-relevants message line.
         A non relevant line can be a commented one, or a to short one.
-        Available comment patterns are: // and #
+
+        Available comment patterns are:  `// and #`
+
         A relevant commit line must contain:
+
         - at least 2 words for formal
         - at least 3 words for keywords
 
         Args:
-            RawChangelogMessage: The full raw changelog (merged commit-history)
+            RawChangelogMessage: The full raw changelog to be processed
         Returns:
-            self class for convenience
+            self for convenience
         """
 
         Lines2ndRound = []
 
         for line in RawChangelogMessage.split("\n"):
             lineWordsCount = len(line.split())
-            if (lineWordsCount > 1) and (not re.match(self.checkCommentPattern, line)):
+            if (lineWordsCount > 1) and (not match(self.checkCommentPattern, line)):
                 if self._ProcessLineMain(line) is True:
                     continue
-                elif lineWordsCount > 2:
+                if lineWordsCount > 2:
                     Lines2ndRound.append(line)
 
         for line in Lines2ndRound:
@@ -257,7 +366,8 @@ class ChangeLogFactory:
         """
         full_changelog = ""
         for formater in sorted(self.ar_Formater.values(), key=lambda x: x.priority, reverse=True):
-            if (include_unknown is False) and (isinstance(formater, ChangeLogFormater_others)):
+            # missing mypy coverage here because of internal bad isinstance() handling
+            if (include_unknown is False) and (isinstance(formater, ChangelogFormater_others)):
                 continue
             full_changelog = full_changelog + formater.Render()
 
@@ -294,32 +404,34 @@ for RecordType, Config in {
                 ),
     "wip":      (   0,  ["temp", ],
                     "Work in progress changes :construction: :",
-                ), 
-    "doc":     (   0,  [ "doc", "manual"], 
+                ),
+    "doc":     (   0,  [ "doc", "manual"],
                     "Documentations :book: :",
                 ),
     "style":    (   5,  ["beautify", ],
-                    "Style :art: :", 
+                    "Style :art: :",
                 ),
     "refactor": (   0,  [],
                     "Refactorings :recycle: :"
                 ),
-    "ci":       (   0,  ["jenkins", "git"], 
+    "ci":       (   0,  ["jenkins", "git"],
                     "Continuous Integration :cyclone: :"
                 ),
-    "test":     (   -5, ["unittest", "check", "testing"], 
+    "test":     (   -5, ["unittest", "check", "testing"],
                     "Testings :vertical_traffic_light: :"
                 ),
-    "build":    (   0,  ["compile", "version"], 
+    "build":    (   0,  ["compile", "version"],
                     "Builds :package: :"
                 ),
     # fmt: on
 }.items():
     # then we instantiate all of them
-    _name = f"ChangeLogFormater_{RecordType}"
-    _tmp = globals()[_name] = type(
+    _name = f"ChangelogFormater_{RecordType}"
+
+    # can not change globals definition so mypy will keep complaining
+    _tmp = type(
         _name,
-        (ChangeLogFormater,),
+        (ChangelogFormater,),
         {
             "prefix": RecordType,
             "title": Config[2],
@@ -327,17 +439,19 @@ for RecordType, Config in {
             "priority": Config[0],
         },
     )
-    ChangeLogFactory.ar_FormaterKlass.add(_tmp)
+    globals()[_name] = _tmp
+    cast(ChangelogFactory[ChangelogFormater], ChangelogFactory).RegisterBaseFormater(_tmp)
+    cast(ChangelogFactory[ChangelogFormater], ChangelogFactory).ar_SavedFormaterKlass.add(_tmp)
 
 
-@ChangeLogFormaterRecordType
-class ChangeLogFormater_revert(ChangeLogFormater):
+@_ChangelogFormaterRecordType
+class ChangelogFormater_revert(ChangelogFormater):
     """Revert scope formater"""
 
-    prefix: str = "revert"
-    title: str = "Reverts :back: :"
-    keywords: list[str] = ["revert", "fallback"]
-    priority: int = 0
+    prefix: ClassVar[Optional[str]] = "revert"
+    title: ClassVar[Optional[str]] = "Reverts :back: :"
+    keywords: ClassVar[Optional[List[str]]] = ["revert", "fallback"]
+    priority: ClassVar[int] = 0
 
     def RenderLines(self) -> str:
         """Render all lines
@@ -350,11 +464,11 @@ class ChangeLogFormater_revert(ChangeLogFormater):
         return full_lines
 
 
-@ChangeLogFormaterRecordType
-class ChangeLogFormater_others(ChangeLogFormater):
+@_ChangelogFormaterRecordType
+class ChangelogFormater_others(ChangelogFormater):
     """Others / unknown scope formater"""
 
-    prefix: str = "other"
-    title: str = "Others :question: :"
-    keywords: list[str] = [""]
-    priority: int = -20
+    prefix: ClassVar[Optional[str]] = "other"
+    title: ClassVar[Optional[str]] = "Others :question: :"
+    keywords: ClassVar[Optional[List[str]]] = [""]
+    priority: ClassVar[int] = -20
